
;..........................................................................................................................................;------------------------------------------------------------------------
; EQUATE.INC
;------------------------------------------------------------------------
; Global EQUATE listing/usage
;------------------------------------------------------------------------
; Note: All MASK definitions are intended to be OR'ed with a register value
;       to SET a bit and AND'ed with the complement of the MASK (~MASK) to
;       CLEAR a bit.
;------------------------------------------------------------------------
LOWER_4K:               EQU 0       ; Lower 4K of EPROM
UPPER_4K:               EQU 1000h   ; Upper 4K of EPROM
                                    ; See PORT_PWR.ASM
MAX_RAM:                EQU FFh     ; MAX size of local SRAM

; CY7C66000 I/O Register Equates (see [3] Table 6-1)
REG_PORT_0:             EQU 00h
REG_PORT_1:             EQU 01h
REG_PORT_2:             EQU 02h
REG_PORT_3:             EQU 03h
REG_PORT_0_INT_ENABLE:  EQU 04h
REG_PORT_1_INT_ENABLE:  EQU 05h
REG_PORT_2_INT_ENABLE:  EQU 06h
REG_PORT_3_INT_ENABLE:  EQU 07h
REG_GPIO_CONFIG:        EQU 08h
REG_HAPI_I2C_CONFIG:    EQU 09h
REG_A_ADDRESS:          EQU 10h
REG_EP_A0_COUNT:        EQU 11h
REG_EP_A0_MODE:         EQU 12h
REG_EP_A1_COUNT:        EQU 13h
REG_EP_A1_MODE:         EQU 14h
REG_EP_A2_COUNT:        EQU 15h
REG_EP_A2_MODE:         EQU 16h
REG_STAT_CTRL:          EQU 1Fh
REG_GLOBAL_INT_ENABLE:  EQU 20h
REG_EP_INT_ENABLE:      EQU 21h
REG_TIMER_LSB:          EQU 24h
REG_TIMER_MSB:          EQU 25h
REG_WDT_CLEAR:          EQU 26h
REG_I2C_STAT_CTRL:      EQU 28h
REG_I2C_DATA:           EQU 29h
REG_DAC_DATA:           EQU 30h
REG_DAC_INT_ENABLE:     EQU 31h
REG_DAC_INT_POLARITY:   EQU 32h
REG_DAC_ISINK_0:        EQU 38h
REG_DAC_ISINK_1:        EQU 39h
REG_DAC_ISINK_2:        EQU 3Ah
REG_DAC_ISINK_3:        EQU 3Bh
REG_DAC_ISINK_4:        EQU 3Ch
REG_DAC_ISINK_5:        EQU 3Dh
REG_DAC_ISINK_6:        EQU 3Eh
REG_DAC_ISINK_7:        EQU 3Fh
REG_B_ADDRESS:          EQU 40h
REG_EP_B0_COUNT:        EQU 41h
REG_EP_B0_MODE:         EQU 42h
REG_EP_B1_COUNT:        EQU 43h
REG_EP_B1_MODE:         EQU 44h
REG_CONNECT_STAT:       EQU 48h         ; set = connected
REG_PORT_ENABLE:        EQU 49h         ; set = enabled
REG_PORT_SPEED:         EQU 4Ah         ; set bits = low speed device
REG_PORT_CTRL_LOW:      EQU 4Bh
REG_PORT_CTRL_UP:       EQU 4Ch
REG_PORT_SUSPEND:       EQU 4Dh         ; set = suspended
REG_PORT_RESUME:        EQU 4Eh         ; set = resume detected
REG_PORT_FORCE_LOW1_4:  EQU 51h
REG_PORT_FORCE_LOW5_7:  EQU 52h
REG_TEST:               EQU F0h
REG_POR_CONFIG:         EQU F1h
REG_HUB_TEST:           EQU F2h
REG_DEVICE_ID:          EQU F3h
REG_MFG_ID:             EQU F4h
REG_PROC_STAT:          EQU FFh

;------------------------------------------------------------------------
; Register specific bit masks
;------------------------------------------------------------------------
; Interrupt enable masks
; for REG_GLOBAL_INT_ENABLE
MASK_128US:             EQU 02h
MASK_1024US:            EQU 04h
MASK_HUB:               EQU 08h
MASK_DAC:               EQU 10h
MASK_GPIO:              EQU 20h
MASK_I2C:               EQU 40h

; for REG_EP_INT_ENABLE
MASK_EPA0:              EQU 01h
MASK_EPA1:              EQU 02h
MASK_EPA2:              EQU 04h
MASK_EPB0:              EQU 08h
MASK_EPB1:              EQU 10h

; for REG_EP_{}_MODE Endpoint Mode Register control Modes
; Both EP0 and EP1 mode registers
; Controls how the endpoint responds to USB bus traffic
MASK_MODE:              EQU 00001111b   ; mode[3:0] bits
;----------------------------------------------------------------
; applicable modes                          SETUP   IN      OUT
;-------------------                       ------   ----    -----
DISABLED:               EQU 00h         ;   ignore  ignore  ignore
NAK_IN_OUT:             EQU 01h         ;   accept  NAK     NAK
STATUS_OUT_ONLY:        EQU 02h         ;   accept  STALL   check
STALL:                  EQU 03h         ;   accept  STALL   STALL
IGNORE:                 EQU 04h         ;   accept  ignore  ignore
ISO_OUT:                EQU 05h         ;   ignore  ignore  always
STATUS_IN_ONLY:         EQU 06h         ;   accept  TX0     STALL
ISO_IN:                 EQU 07h         ;   ignore  TX cnt  ignore
NAK_OUT:                EQU 08h         ;   ignore  ignore  NAK
ACK_OUT:                EQU 09h         ;   ignore  ignore  ACK
NAK_OUT_STATUS_IN:      EQU 0Ah         ;   accept  TX0     NAK
ACK_OUT_STATUS_IN:      EQU	0Bh         ;   accept  TX0     ACK
NAK_IN:                 EQU	0Ch	        ;   ignore  NAK     ignore
ACK_IN:                 EQU 0Dh         ;   ignore  TX cnt  ignore
NAK_IN_STATUS_OUT:      EQU 0Eh         ;   accept  NAK     check
ACK_IN_STATUS_OUT:      EQU 0Fh	        ;   accept  TX cnt  check

; endpoint 0 only mode registers
MASK_TOKEN:             EQU 11100000b   ; token status bits
MASK_ACK:               EQU 00010000b   ; ACK received from host ;*? NOT DOCUMENTED(data sheet)!!
OUT_TOKEN:              EQU 00100000b   ; OUT token received
IN_TOKEN:               EQU 01000000b   ; IN token received
SETUP_TOKEN:            EQU 10000000b   ; SETUP token received

; for REG_EP_{}_COUNT endpoint count registers
MASK_COUNT:             EQU 00001111b   ; count[3:0] bits
MASK_DATAVALID:         EQU 01000000b   ; data valid (OUT and SETUP)
MASK_DATATOGGLE:        EQU 10000000b   ; Data 0/1 bit

; for REG_x_ADDR (x = A or B)
MASK_ADDRESS:           EQU 01111111b   ; REG_x_ADDR (x = A or B)

; for REG_STAT_CTRL (USB status and control)
MASK_UPSTREAM_CONTROL:      EQU 00000111b ; Upstream control bits 0-3

;(BETA_BUG)*********************************
MASK_BETA_BUG:              EQU 11000000b   ; mask ENPsize and 5 endpoints
;*******************************************

MASK_UPSTREAM_BUS_ACTIVITY: EQU 00001000b ; Bus Activity "sticky" bit

MASK_FORCE_UPSTREAM_K:      EQU 00000010b ; UD+ low, UD- high


; for REG_HUB_PORT_CTRL Local control bits for downstream port state forcing
MASK_NOT_FORCING:       EQU 00000000b   ;
MASK_FORCE_DIFF_1:      EQU 00000001b   ; force differential 1 downstream port(s)
MASK_FORCE_DIFF_0:      EQU 00000010b   ; force differential 0 downstream port(s)
MASK_FORCE_SE0:         EQU 00000011b   ; force SE0 state on downstream port(s)
MASK_FORCE_FS_J:        EQU MASK_FORCE_DIFF_1 ; Full-speed J
MASK_FORCE_FS_K:        EQU MASK_FORCE_DIFF_0 ; Full-speed K
MASK_FORCE_LS_J:        EQU MASK_FORCE_DIFF_0 ; Low-speed J
MASK_FORCE_LS_K:        EQU MASK_FORCE_DIFF_1 ; Low-speed K


; for REG_PROC_STAT processor status register
MASK_RUN:               EQU 01h         ; bit 0
MASK_SINGLE_STEP:       EQU 02h         ; bit 1
MASK_SUSPEND:           EQU 08h         ; bit 3
MASK_BUS_RESET_INT:     EQU 20h         ; bit 5

; wHubStatus bit masks
MASK_LOCAL_POWER_STATUS:EQU 01h
MASK_OVER_CURRENT:      EQU 02h

; wPortStatus and wPortChange bit masks (r.e. [2] Tables 11-15 and 11-16)
; HUB_PortStatusx_LSB and HUB_PortChangex_LSB........ (x = 0-3)
MASK_PORT_CONNECTION:   EQU 01h
MASK_PORT_ENABLE:       EQU 02h
MASK_PORT_SUSPEND:      EQU 04h
MASK_PORT_OVER_CURRENT: EQU 08h
MASK_PORT_RESET:        EQU 10h
; HUB_PortStatusx_MSB........
MASK_PORT_POWER:        EQU 01h
MASK_PORT_LOW_SPEED:    EQU 02h

; BITMAP_HUB_EP1 masks
MASK_BITMAP_PORTS:      EQU 00011110b
MASK_BITMAP_HUB:        EQU 00000001b
MASK_HUB_CHANGE:        EQU 00000001b
MASK_PORT1_CHANGE:      EQU 00000010b
MASK_PORT2_CHANGE:      EQU 00000100b
MASK_PORT3_CHANGE:      EQU 00001000b
MASK_PORT4_CHANGE:      EQU 00010000b
MASK_PORT5_CHANGE:      EQU 00100000b
MASK_PORT6_CHANGE:      EQU 01000000b
MASK_PORT7_CHANGE:      EQU 10000000b

; GPIO bit masks
MASK_BIT_0:             EQU 00000001b
MASK_BIT_1:             EQU 00000010b
MASK_BIT_2:             EQU 00000100b
MASK_BIT_3:             EQU 00001000b
MASK_BIT_4:             EQU 00010000b
MASK_BIT_5:             EQU 00100000b
MASK_BIT_6:             EQU 01000000b
MASK_BIT_7:             EQU 10000000b

;SYSTEM STATES ----------------------------------------------------------
DEFAULT_STATE:      EQU 01h     ; default = 1, a state should NEVER = 0
;------------------------------------------------------------------------
; Valid device (hub) states (STATE_HUB) (0 states are invalid)
; r.e. [1] Table 9-1 "Visable Device States", page 167
; Note: USB does not assign specific values to the states. Assignment values are
;       user dependent but must be unique.
;------------------------------------------------------------------------
HUB_DEFAULT:        EQU 01h     ; root port reset detected, default state
                                ; address not assigned and pending req. for device descriptor
HUB_ADDRESS:        EQU 02h     ; host assigned an address
HUB_CONFIGURED:     EQU 03h     ; config's requested and sent, configuration value
                                ; assigned by host
                                ; !!!enumeration complete!!!
MASK_HUB_SUSPEND:   EQU 80h     ; continuous idle for >3.0ms
                                ; MUST OR MASK with curent state >= DEFAULT
                                ; Useful for INT's that must know if INT takes place
                                ; while hub is suspended.
HUB_AWAKE:          EQU 03h     ; same as configured
                                ; MUST AND ~MASK with current state if configured
;------------------------------------------------------------------------
; Configuration settings used by SET_CONFIGURATION request
;------------------------------------------------------------------------
UNCONFIGURED:	    EQU	0	; Unconfigured value sent by host to device
CONFIGURED:	    EQU	1	; Configured value sent by host to device

;------------------------------------------------------------------------
; Downstream Port States (STATE_HUB_PORTx, see MALLOC.INC)
; 0 is an invalid state!
;------------------------------------------------------------------------
;PORT_IDLE:         EQU 1       ; Connection has not been detected by hub repeater
PORT_POWERED_OFF:   EQU 1       ; ClearPortFeature(PORT_POWER) or root port reset detect
PORT_DISCONNECTED:  EQU 2       ; SetPortFeature(PORT_POWER) or disconnect event
PORT_DISABLED:      EQU 3       ; detection of connect event
                                ; or ClearPortFeature(PORT_ENABLE) or frame error (fr_error)
PORT_ENABLED:       EQU 4       ; SetPortFeature(PORT_ENABLE) or SetPortFeature(PORT_RESET)
PORT_SUSPENDED:     EQU 5       ; SetPortFeature(PORT_SUSPEND)
;------------------------------------------------------------------------

;------------------------------------------------------------------------
; States of STATE_RESUME_PENDING
;------------------------------------------------------------------------
RESUME_COMPLETE:    EQU 1       ; default resume state
RESUME_START:       EQU 2       ; port initiated start of resume
RESUME_OVERRIDE:    EQU 3       ; firmware override of hub HW forcing control

                                        ; ready for enumeration
CONFIG_STAT_CTRL:       EQU 0
CONFIG_GLOBAL_INT:      EQU 9	  ; enable Hub interrupt and the USB Bus Reset Intr
CONFIG_EP_INT:          EQU ( MASK_EPB0 )

