;FILE HIDMAIN.ASM
;this file is the integration point between the hub code and the hid code
;-----------------------------------------------------------------------------
ram_base:	equ	BASE_HID1        ;locate the start of hid ram at its base, as
                                ;defined in malloc.inc



ROM_HID_START:
;include all "inc" files used by the code

    include "regs.inc"
    include "usb.inc"
    include "macros.inc"

HID_PORT:                EQU 4       ; HID is 4th port (of 0 to 4: 5th port)

;we need to use all 3 endpoints for the HID.

EP_INTERRUPTS: equ EP_A0_A1_A2



;include all code which resides in lower 4k now.

usbmain_ram_base:		EQU	BASE_HID1


HID_INIT:
        MOV     A, DEFAULT_STATE    ;init hid state to default
    	MOV     [STATE_HID], A
    	disable_hid_ep0_int
    	disable_hid_ep1_int
    	disable_hid_ep2_int
    	mov A,0
    	iowr USB_Device_Address     ;zero address reg
   	mov A, DISABLED         	; disable endpoint one
	iowr EP_A0_Mode
   	mov A, DISABLED         	; disable endpoint one
	iowr EP_A1_Mode
   	mov A, DISABLED         	; disable endpoint two
   	iowr EP_A2_Mode
   	call Init           ; perform initialization on variables
        ret



TASK_HID:
; This is a main loop task that waits for the host to power then
; enable the "virtual port" for the hid before the default endpoint and
; address (hardware) for the hid is enabled.
;------------------------------------------------------------------------
    MOV     A, HID_PORT                  ; A = HID PORT
    MOV     X,A                         ; get state of HID
    MOV     A,[X+HUB_PORT_STATE_BASE]   ;
    CMP     A,[STATE_HID]                ; compare with last state
    JNZ     HIDT_000                     ; if no change, test for enabled
    CMP     A,PORT_ENABLED
    JNZ     TASK_HID_END
    JMP     HID_MAIN
HIDT_000:
    CMP     A,PORT_POWERED_OFF          ; there is a change, see if powering off
    JNZ     HIDT_00
    MOV     [STATE_HID],A
    CALL    HID_INIT                     ;  if yes, initialize everything
    JMP     TASK_HID_END                  ;  and set next state
HIDT_00:
    CMP     A,PORT_DISCONNECTED         ;  if disconnected, (powered on)
    JNZ     HIDT_01
    MOV     A,PORT_DISABLED
    MOV     [STATE_HID],A
    MOV     A,HID_PORT
    ASL
    ASL
    ADD     A,HUB_PORT_STATUS_BASE
    MOV     X,A
    MOV     A,HID_PORT
    CALL    SET_PORT_CONNECT             ;  move to connected
    JMP     TASK_HID_END
HIDT_01:
    CMP     A,PORT_ENABLED              ;  if enabled
    JNZ     HIDT_02
    PUSH    A
    MOV     A,[STATE_HID]                ;  if enabled coming out of suspend
    CMP    A,PORT_SUSPENDED
    POP   A
    MOV    [STATE_HID],A
    JZ    TASK_HID_END                   ;  just exit

    CALL    HID_INIT                     ;  if yes, initialize everything
    MOV     A,PORT_ENABLED
    MOV     [STATE_HID],A
    iord    EP_A0_Mode                  ; else unlock Mode register
    mov     A, SETUP		            ; enable endpoint zero setup
    iowr    EP_A0_Mode
    mov      A,ADDRESS_ENABLE_BIT
    iowr    USB_Device_Address
    enable_hid_ep0_int
    JMP     TASK_HID_END


HIDT_02:
    CMP     A,PORT_DISABLED              ;  if disabled
    JNZ     HIDT_03
    MOV    [STATE_HID],A                  ;simply save the state change
    JZ    TASK_HID_END                    ;  and exit

 HIDT_03:
    CMP     A,PORT_SUSPENDED
    JNZ     TASK_HID_END
    MOV     A,0                         ;keyboard "suspended"

HID_MAIN:
    call Main_Machine

TASK_HID_END:
    RET



INT_GPIO:
        RETI


;------------------------------------------------------------------------
INIT_GPIO:

    RET



INCLUDE "./hidsrc/hid_usb.asm"

