;------------------------------------------------------------------------
; HUB_EP0C.ASM
; Sub-Module to HUB_EP0.ASM
;------------------------------------------------------------------------

;========================================================================
PROC_HT_IN_0:
; [2] 11.12.2 Class-specific Requests
; Class (hub specific) host-to-hub (IN) request
; [bmRequestType] ClearHubFeature, SetHubDescriptor, SetHubFeature
; [bRequest] CLEAR_FEATURE, SET_DESCRIPTOR, SET_FEATURE
;========================================================================
    MOV     A, [HUB_bRequest]
    CMP     A, CLEAR_FEATURE
    JZ      CLEAR_HUB_FEATURE
    CMP     A, SET_DESCRIPTOR
    JZ      SET_HUB_DESCRIPTOR
    CMP     A, SET_FEATURE
    JZ      SET_HUB_FEATURE
    JMP     HUB_EP0_STALL

;........................................................................
CLEAR_HUB_FEATURE:
; resets a value reported in hub status
; r.e. [2] 11.12.2.1
;........................................................................
    MOV     A, [HUB_wValue_MSB]
    CMP     A, 0                        ; ensure MSB = 0
    JNZ     HUB_EP0_STALL
    MOV     A, [HUB_wValue_LSB]
    CMP     A, C_HUB_LOCAL_POWER
    JZ      CLEAR_C_HUB_LOCAL_POWER
    CMP     A, C_HUB_OVER_CURRENT
    JZ      CLEAR_C_HUB_OVER_CURRENT
    JMP     HUB_EP0_STALL

;........................................................................
SET_HUB_DESCRIPTOR:
; optional (not supported)
;........................................................................
    JMP     HUB_EP0_STALL

;........................................................................
SET_HUB_FEATURE:
; r.e. [2] 11.12.2.8
;........................................................................
;    MOV     A, [HUB_wValue_MSB]
;    CMP     A, 0
;    JNZ     HUB_EP0_STALL
;    MOV     A, [HUB_wValue_LSB]
;    CMP     A, C_HUB_LOCAL_POWER
;    JZ      SET_C_HUB_LOCAL_POWER
;    CMP     A, C_HUB_OVER_CURRENT
;    JZ      SET_C_HUB_OVER_CURRENT
    JMP     HUB_EP0_STALL

;........................................................................
;*? more here? [2] pg 263
CLEAR_C_HUB_LOCAL_POWER:
;........................................................................
;*? test if if hub is self (locally) powered (NOT bus powered)
;*? test HUB_CONFIG_DESCRIPTOR (bmAttributes)

    MOV     A, 0
    MOV     [HUB_wHubStatus_MSB], A
    MOV     A, [HUB_wHubChange_LSB]
    AND     A, ~MASK_LOCAL_POWER_STATUS
    MOV     [HUB_wHubChange_LSB], A
    JMP     NO_DATA_CONTROL

;........................................................................
;*? more here? [2] pg 263
CLEAR_C_HUB_OVER_CURRENT:
;........................................................................
    MOV     A, 0
    MOV     [HUB_wHubStatus_MSB], A
    MOV     A, [HUB_wHubStatus_LSB]
    AND     A, ~MASK_OVER_CURRENT
    MOV     [HUB_wHubStatus_LSB], A
    JMP     NO_DATA_CONTROL

;........................................................................
;*? more here? [2] pg 274
SET_C_HUB_LOCAL_POWER:
;........................................................................
    MOV     A, 0
    MOV     [HUB_wHubStatus_MSB], A
    MOV     A, [HUB_wHubStatus_LSB]
    OR      A, MASK_LOCAL_POWER_STATUS
    MOV     [HUB_wHubStatus_LSB], A
    JMP     NO_DATA_CONTROL

;........................................................................

SET_C_HUB_OVER_CURRENT:
;........................................................................
    MOV     A, 0
    MOV     [HUB_wHubStatus_MSB], A
    MOV     A, [HUB_wHubStatus_LSB]
    OR      A, MASK_OVER_CURRENT
    MOV     [HUB_wHubStatus_LSB], A
    JMP     NO_DATA_CONTROL

;------------------------------------------------------------------------
GET_PORT_FEATURE_INDEX:
; table search for HUB_wValue (feature selector for port specific requests)
; On EXIT, A = index into TABLE_PORT_FEATURE for valid feature
;            = FFh for invalid feature
;------------------------------------------------------------------------
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
PF_LOOK:
    INDEX   TABLE_PORT_FEATURE          ; fetch feature
    CMP     A, [HUB_wValue_LSB]         ; = wValue_LSB? (match)
    JZ      PF_FOUND                    ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      PF_NOT_FOUND                ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     PF_LOOK                     ; look at next request for match
PF_FOUND:
    MOV     A, X                        ; A = index (valid)
PF_NOT_FOUND:                           ; else A = FFh (invalid)
    RET

;========================================================================
PROC_HT_IN_1:
; [2] 11.12.2 Class-specific Requests
; Class (port specific) host-to-hub (IN) request
; [bmRequestType] ClearPortFeature, SetPortFeature
; [bRequest] CLEAR_FEATURE, SET_FEATURE
;========================================================================
    MOV     A, [HUB_wIndex_MSB]         ; wIndex = Port number
    CMP     A, 0                        ; ensure MSB = 0
    JNZ     PROC_HT_IN_1_EXIT           ; if not, exit
    MOV     A, [HUB_wIndex_LSB]         ; ensure LSB <= MAX_PORTS
    CMP     A, MAX_PORTS+1
    JNC     PROC_HT_IN_1_EXIT           ; if not, exit
    CMP     A, 0                        ; ensure LSB != 0, port must be non-zero
    JZ      PROC_HT_IN_1_EXIT           ; if zero, exit
    DEC     A                           ; make port number 0 based
    MOV     [HUB_PORT], A               ; save the port number
    MOV     A, [HUB_bRequest]           ; get bRequest
    CMP     A, CLEAR_FEATURE
    JZ      CLEAR_PORT_FEATURE          ; clear port feature
    CMP     A, SET_FEATURE
    JZ      SET_PORT_FEATURE            ; set port feature
PROC_HT_IN_1_EXIT:
    JMP     HUB_EP0_STALL

;........................................................................
CLEAR_PORT_FEATURE:
; r.e. [2] 11.12.2.2
;........................................................................
    MOV     A, [HUB_wValue_MSB]         ; ensure wValue MSB = 0
    CMP     A, 0
    JNZ     HUB_EP0_STALL               ; if not, exit
    CALL    GET_PORT_FEATURE_INDEX      ; get index into port feature table
    CMP     A, INVALID_FEATURE          ; valid feature?
    JZ      HUB_EP0_STALL               ; if not, exit
    ASL                                 ; *2
    JACC    JMP_TABLE_CLEAR_PORT_FEATURE; jump to request processor

;........................................................................
CLEAR_PORT_CHANGE_BIT:
; Clear a bit in wPortChange
; On entry [A]    contains port number
;          [TEMP] contains mask to AND with current wPortChange setting
;........................................................................
    ASL                                 ; A*2
    ASL                                 ; A*4
    ADD     A, 2                        ; A+2
    SWAP    A, X                        ; X = (X*4)+2 (offset into wPortChange base)
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortChange
    AND     A, [TEMP]                   ; clear the change bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save change
    RET

;........................................................................
SET_PORT_CHANGE_BIT:
; Set a bit in wPortChange
; On entry [A]    contains port number
;          [TEMP] contains mask to OR with current wPortChange setting
;........................................................................
    ASL                                 ; A*2
    ASL                                 ; A*4
    ADD     A, 2                        ; A+2
    SWAP    A, X                        ; X = (X*4)+2 (offset into wPortChange base)
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortChange
    OR      A, [TEMP]                   ; set the change bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save change
    RET

;........................................................................
CLEAR_PORT_STATUS_BIT:
; Clear a bit in WPortStatus
; On entry [A]    contains port number
;          [TEMP] contains mask to AND with current wPortStatus setting
;........................................................................
    ASL                                 ; A*2
    ASL                                 ; A*4
    SWAP    A, X                        ; X = (X*4)(offset into wPortStatus base)
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortStatus
    AND     A, [TEMP]                   ; clear the status bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save status
    RET

;........................................................................
SET_PORT_STATUS_BIT:
; Set a bit in WPortStatus
; On entry [A]    contains port number
;          [TEMP] contains mask to OR with current wPortStatus setting
;........................................................................
    ASL                                 ; A*2
    ASL                                 ; A*4
    SWAP    A, X                        ; X = (X*4)(offset into wPortStatus base)
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortStatus
    OR      A, [TEMP]                   ; clear the status bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save status
    RET

;........................................................................
CLEAR_PORT_ENABLE:
; r.e. [2] Table 11-1, Port Behavior vs. Signaling
; Port state must be PORT_ENABLED, else is ignored
; Port will transition to PORT_DISABLED state
;........................................................................
    MOV     X, [HUB_PORT]               ; use X as offset into state base
    MOV     A, PORT_DISABLED
    MOV     [X+HUB_PORT_STATE_BASE], A  ; bump state back to DISABLED

    MOV     A, X                        ; A = X = port number
    INDEX   TABLE_PORT_MASK             ; get port enable mask
    CPL     A                           ; complement
    MOV     [TEMP], A                   ; save ~mask to TEMP
    read    REG_PORT_ENABLE             ; read current port enable register
    AND     A, [TEMP]                   ; clear port enable bit
    write_A REG_PORT_ENABLE             ; write back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable

    MOV     A, ~MASK_PORT_ENABLE        ; clear enable bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_STATUS_BIT

    MOV     A, MASK_PORT_ENABLE         ; set enable/disable change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    SET_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
CLEAR_PORT_SUSPEND:
; PORT_RESUME
; This routine starts the port resume signaling to the downstream port by
; saving HUB_PORT to HUB_RESUME_PORT, driving resume (K) on the downstream port,
; then setting FLAG_RESUME_PORT, if the port is in the suspended state.
; TASK_PORT_RESUME handles the timeout for (K) assertion and the removal of (K)
; control, then places the port in the enabled state.
; wPortStatus and wPortChange are handled by TASK_PORT_RESUME.
;........................................................................

    MOV     A, [HUB_PORT]
    CALL    SEND_RESUME
    JMP     NO_DATA_CONTROL            ; get out.
;........................................................................
CLEAR_PORT_OVER_CURRENT:
;........................................................................
    MOV     A, ~MASK_PORT_OVER_CURRENT   ; clear overcurrent bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_STATUS_BIT
    MOV     A, MASK_PORT_OVER_CURRENT    ; set overcurrent change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    SET_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
CLEAR_PORT_POWER:
; r.e. [2] Table 11-1, Port Behavior vs. Signaling
; Port state must be ANY state except PORT_POWERED_OFF, else is ignored
; POWER_OFF_ONE resets state to PORT_POWERED_OFF
;........................................................................
; First clear enable and suspend bits
	MOV	A,[HUB_PORT]	
	INDEX	TABLE_PORT_MASK	
	CPL
	MOV	[TEMP], A
	read	REG_PORT_ENABLE
	AND	A, [TEMP]
	write_A REG_PORT_ENABLE
	read	REG_PORT_SUSPEND
	AND	A, [TEMP]
	write_A REG_PORT_SUSPEND
    MOV     A, [HUB_PORT]
    CALL    POWER_OFF_ONE               ; remove power from selected port, set to POWERED_OFF
	MOV	A, [HUB_PORT]
                                        ; bth -- modify our copy of connect status,
                                        ; since the port is powered off. 
	INDEX	TABLE_PORT_MASK	        	; Index to the table, table is in Hub.asm
    CPL A
	MOV	[TEMP], A
    MOV A,[HUB_LAST_CONNECT_STAT]
    AND A,[TEMP]
    MOV [HUB_LAST_CONNECT_STAT],A


; Clear the corresponding enable bit of the Enable Register, and the suspend bit of the Suspend Register as a part of 
; the power-off routine. Then, bump the port state to PORT_POWERED_OFF state.

    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
CLEAR_C_PORT_CONNECTION:
; Clears wPortChange, Connect Status Change (C_PORT_CONNECTION) bit
;........................................................................
    MOV     A, ~MASK_PORT_CONNECTION    ; clear connect status change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
CLEAR_C_PORT_ENABLE:
; Clears wPortChange, Port Enable/Disable Change (C_PORT_ENABLE) bit
; Port can be in either ENABLED or DISABLED state
;........................................................................
    MOV     A, ~MASK_PORT_ENABLE        ; clear enable/disable change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
CLEAR_C_PORT_SUSPEND:
; Clears wPortChange, Port Suspend Change (C_PORT_SUSPEND) bit
;........................................................................
    MOV     A, ~MASK_PORT_SUSPEND       ; clear suspend change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_CHANGE_BIT

    JMP     NO_DATA_CONTROL            ; get out.
;........................................................................
CLEAR_C_PORT_OVER_CURRENT:
;........................................................................
    MOV     A, ~MASK_PORT_OVER_CURRENT   ; clear overcurent change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.


;........................................................................
CLEAR_C_PORT_RESET:
; Clears wPortChange, Reset Change (C_PORT_RESET) bit
;........................................................................
    MOV     A, ~MASK_PORT_RESET         ; clear reset change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
SET_PORT_FEATURE:
; r.e. [2] 11.12.2.9
;........................................................................
    MOV     A, [HUB_wValue_MSB]         ; ensure wValue MSB = 0
    CMP     A, 0
    JNZ     HUB_EP0_STALL               ; if not, exit
    CALL    GET_PORT_FEATURE_INDEX      ; get index into port feature table
    CMP     A, INVALID_FEATURE          ; valid feature?
    JZ      HUB_EP0_STALL               ; if not, exit
    ASL                                 ; *2
    JACC    JMP_TABLE_SET_PORT_FEATURE  ; jump to request processor

;........................................................................
SET_PORT_ENABLE:
; r.e. [2] Table 11-1, Port Behavior vs. Signaling
; Port state must be PORT_DISABLED, else is ignored
; Port will transition to PORT_ENABLED state
;........................................................................
    MOV     X, [HUB_PORT]               ; use X as offset into state base
    MOV     A, PORT_ENABLED
    MOV     [X+HUB_PORT_STATE_BASE], A  ; bump state to ENABLED

    MOV     A, X                        ; A = X = port number
    INDEX   TABLE_PORT_MASK             ; get port enable mask
    MOV     [TEMP], A                   ; save enable mask to TEMP
    read    REG_PORT_ENABLE             ; read current port enable register
    OR      A, [TEMP]                   ; set port enable bit
    write_A REG_PORT_ENABLE             ; write back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable

    MOV     A, MASK_PORT_ENABLE         ; set enable bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    SET_PORT_STATUS_BIT

    MOV     A, MASK_PORT_ENABLE         ; set enable/disable change bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    SET_PORT_CHANGE_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
SET_PORT_SUSPEND:
;........................................................................
    MOV     X, [HUB_PORT]               ; use X as offset into state base
    MOV     A, PORT_SUSPENDED
    MOV     [X+HUB_PORT_STATE_BASE], A  ; bump state to SUSPENDED

    MOV     A, X                        ; A = X = port number
    INDEX   TABLE_PORT_MASK             ; get port enable mask
    MOV     [TEMP], A                   ; save enable mask to TEMP
    read    REG_PORT_SUSPEND            ; read current port suspend register
    OR      A, [TEMP]                   ; set port enable bit
    write_A REG_PORT_SUSPEND            ; write back to port suspend register

    MOV     A, MASK_PORT_SUSPEND        ; set PORT_SUSPEND status bit
    MOV     [TEMP], A                   ; DO NOT set change bit
    MOV     A, [HUB_PORT]               ; DO NOT report change to host
    CALL    SET_PORT_STATUS_BIT
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
SET_PORT_RESET:
; r.e. [2] 11.6.2 Per Port Reset
; This routine starts the port reset signaling to the downstream port by
; saving HUB_PORT to HUB_RESET_PORT, driving SE0 on the downstream port,
; then setting FLAG_RESET_PORT, if the port is not in either the powered off
; or disconnected states.
; TASK_PORT_RESET then handles the timeout for SE0 assertion and the
; removal of the SE0 mode, and then places the port in the enabled state.
; wPortStatus and wPortChange are handled by both routines.
;........................................................................
    MOV     A, [HUB_PORT]               ; get port number
    MOV     [HUB_RESET_PORT], A         ; save to hub reset port
    MOV     [HUB_PORT_CTRL], A          ; save port to HUB_PORT_CTRL

    set_flag    FLAG_PORT_RESET         ; set the main loop flag for continued(time delay)processing
    MOV     A, MASK_FORCE_SE0           ; force SE0 state
    MOV     [HUB_PORT_ACTION], A        ; save forcing action
    CALL    SET_PORT_CONTROL            ;tpmV3.2 force SE0 PRIOR to disabling
                                        ; the port to suppress potential 
                                        ; babble from the keep-alive signal

	MOV     A, [HUB_PORT]           ; get port number
	INDEX	TABLE_PORT_MASK		; convert it to port bit placement
	CPL	A			; 
	MOV	[TEMP], A
	IORD	REG_PORT_ENABLE		; read the enable bit
	AND	A, [TEMP]
	IOWR	REG_PORT_ENABLE		; mask off the enable bit
	
	IORD	REG_PORT_SUSPEND
	AND	A, [TEMP]
	IOWR	REG_PORT_SUSPEND	; mask off the suspend bit


    MOV     A, MASK_PORT_RESET          ; set port status reset bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    SET_PORT_STATUS_BIT

    MOV     A, ~MASK_PORT_RESET         ; clear port change reset bit
    MOV     [TEMP], A
    MOV     A, [HUB_PORT]
    CALL    CLEAR_PORT_CHANGE_BIT

    MOV     A, 0
    MOV     [HUB_PORT_RESET_COUNT], A   ; clear the count, 1.024ms timer increments this count

; For Set_Port_Reset, should clear the STATE_RESUME_PENDING since no resume signalling is needed.

	MOV	A,1
	MOV     [STATE_RESUME_PENDING], A      
    JMP     NO_DATA_CONTROL            ; get out.

;........................................................................
SET_PORT_POWER:
; r.e. [2] Table 11-1, Port Behavior vs. Signaling
; Port state must be PORT_POWERED_OFF, else is ignored
; POWER_ON_ONE sets state to DISCONNECTED
;........................................................................
; test HUB_PORT_POWER_CTRL (HUB_CLASS_DESCRIPTOR (PortPwrCtrlMask)) for port bit enables
    MOV     A, [HUB_PORT]
    CALL    POWER_ON_ONE                ; apply power to selected port, sets state to DISCONNECTED

; Next section was added to pass Ch.11 test (Test 5.3.5)
	MOV	A, [HUB_PORT]
	INDEX	TABLE_PORT_MASK		; Index to the table, table is in Hub.asm
	MOV	[TEMP], A
	IORD	REG_CONNECT_STAT	; check if a device is connected
	AND	A, [TEMP]		; if yes, prepare to set the PORT_CONNECTION bit
	JZ	SKIP_PORT_CONNECT_SET
    MOV A,[HUB_LAST_CONNECT_STAT]
    OR  A,[TEMP]
    MOV [HUB_LAST_CONNECT_STAT],A
	DEC	X			; Push X back to LSB
	MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortStatusX_LSB
	OR      A, MASK_PORT_CONNECTION      ; set por connection bit
	MOV     [X+HUB_PORT_STATUS_BASE], A  ; save status
    MOV     A,[X+(HUB_PORT_STATUS_BASE+2)];set change bit too
    OR      A,MASK_PORT_CONNECTION
    MOV     [X+(HUB_PORT_STATUS_BASE+2)],A;   

    ;9/10/99 added
	MOV	A, [HUB_PORT]
	INDEX	TABLE_PORT_MASK		            ; Index to the table, table is in Hub.asm
	MOV	[TEMP], A
	IORD	REG_PORT_SPEED  	            ; check speed
    	MOV     [SHADOW_PORT_SPEED], A          ; save to shadow register SHADOW_PORT_SPEED
	AND	A, [TEMP]		                    ; if yes, prepare to set the PORT_LOW_SPEED bit
	JZ	SKIP_PORT_SPEED_SET
	MOV     A, [X+(HUB_PORT_STATUS_BASE+1)] ; get wPortStatusX_MSB
	OR      A, MASK_PORT_LOW_SPEED          ; set por speed bit
	MOV     [X+(HUB_PORT_STATUS_BASE+1)], A  ; save status
SKIP_PORT_SPEED_SET:
    
	
SKIP_PORT_CONNECT_SET:	
    JMP     NO_DATA_CONTROL            ; get out.

;========================================================================
PROC_HT_OUT_0:
; [2] 11.12.2 Class-specific Requests
; Class (hub specific) hub-to-host (OUT) request
; [bmRequestType] GetHubDescriptor, GetHubStatus
; [bRequest] GET_DESCRIPTOR, GET_STATUS
;========================================================================
    MOV     A, [HUB_bRequest]           ; get bRequest
    CMP     A, GET_DESCRIPTOR
    JZ      GET_HUB_DESCRIPTOR          ; get hub descriptor request
    CMP     A, GET_STATUS
    JZ      GET_HUB_STATUS              ; get hub status request
PROC_HT_OUT_0_EXIT:
    JMP     HUB_EP0_STALL

;........................................................................
GET_HUB_DESCRIPTOR:
; r.e. [2] 11.12.2.4
;........................................................................
	MOV     A, (HUB_CLASS_DESCRIPTOR_END - HUB_CLASS_DESCRIPTOR); get num bytes to send
    MOV     [HUB_DATA_COUNT], A         ; save the count
    MOV     A, (HUB_CLASS_DESCRIPTOR - TABLE_DESCRIPTOR) ; get start index
    JMP     SEND_DESCRIPTOR             ; send the device descriptor

;........................................................................
GET_HUB_STATUS:
; r.e. [2] 11.12.2.5
HUB_STATUS_SIZE:   EQU 4
;........................................................................
    MOV     A, HUB_STATUS_BASE          ; get base address of hub status
    MOV     [HUB_DATA_START], A         ; save as a pointer to start of hub status data
    MOV     A, HUB_STATUS_SIZE          ; setup to send count
    MOV     [HUB_DATA_COUNT], A
    MOV     [HUB_BYTE_COUNT], A
    CALL    LOAD_STATUS                 ; load the status data (wHubStatus & wHubChange)
    JMP     HUB_ACK_IN_WITH_COUNT

;========================================================================
PROC_HT_OUT_1:
; [2] 11.12.2 Class-specific Requests
; Class (bus/port specific) hub-to-host (OUT) request
; [bmRequestType] GetBusState, GetPortStatus
; [bRequest] GET_STATE, GET_STATUS
;========================================================================
    MOV     A, [HUB_wIndex_MSB]         ; wIndex = Port number
    CMP     A, 0                        ; ensure MSB = 0
    JNZ     PROC_HT_OUT_1_EXIT          ; if not, exit
    MOV     A, [HUB_wIndex_LSB]         ; ensure LSB <= MAX_PORTS
    CMP     A, MAX_PORTS+1
    JNC     PROC_HT_OUT_1_EXIT          ; if not, exit
    CMP     A, 0                        ; ensure LSB != 0, port must be non-zero
    JZ      PROC_HT_OUT_1_EXIT          ; if zero, exit
    DEC     A                           ; make port number 0 based
    MOV     [HUB_PORT], A               ; save the port number
    MOV     A, [HUB_bRequest]           ; get bRequest
    CMP     A, GET_STATE
    JZ      GET_BUS_STATE               ; get bus state request
    CMP     A, GET_STATUS
    JZ      GET_PORT_STATUS             ; get port status request
PROC_HT_OUT_1_EXIT:
    JMP     HUB_EP0_STALL

;........................................................................
GET_BUS_STATE:
; Not supported by Hardware (optional per-port diagnostic request)
;........................................................................
    JMP     HUB_EP0_STALL

;........................................................................
GET_PORT_STATUS:
; r.e. [2] 11.12.2.6
PORT_STATUS_SIZE:   EQU 4
;........................................................................
    MOV     A, [HUB_PORT]               ; restore port number
    ASL                                 ; A*2
    ASL                                 ; A*4
    ADD     A, HUB_PORT_STATUS_BASE     ; add to wPortStatus base address
    MOV     [HUB_DATA_START], A         ; save as a pointer to start of hub status data
    MOV     A, PORT_STATUS_SIZE         ; setup to send count
    MOV     [HUB_DATA_COUNT], A
    MOV     [HUB_BYTE_COUNT], A
    CALL    LOAD_STATUS                 ; send the status data (wPortStatus & wPortChange)
    JMP     HUB_ACK_IN_WITH_COUNT

; Unsupported Requests (Return Stall)
;*? verify ALL below!
;-------------------------------------- Invalid clear port features?
CLEAR_PORT_CONNECTION:
CLEAR_PORT_LOW_SPEED:
;-------------------------------------- Invalid set port features?
SET_PORT_CONNECTION:
SET_PORT_OVER_CURRENT:
SET_PORT_LOW_SPEED:
SET_C_PORT_CONNECTION:
SET_C_PORT_ENABLE:
SET_C_PORT_SUSPEND:
SET_C_PORT_OVER_CURRENT:
SET_C_PORT_RESET:
    JMP     HUB_EP0_STALL

CLEAR_PORT_RESET:
    ;bth added as requested by lxa. Some hosts do not like it if
    ;this request is stalled.  Ack it instead but do nothing about it.
    JMP     NO_DATA_CONTROL            ; get out.


; END HUB_EP0C.ASM -------------------------------------------------------