;------------------------------------------------------------------------
; HUB_EP0S.ASM
; Sub-Module to HUB_EP0.ASM
;         
;------------------------------------------------------------------------

;========================================================================
PROC_ST_IN_0:
; Standard device specific host-to-device (IN) request
; [bRequest] CLEAR_FEATURE, SET_ADDRESS, SET_CONFIGURATION, SET_DESCRIPTOR, SET_FEATURE
;========================================================================
;-- table search for HUB_bRequest
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
SI0_LOOK:
    INDEX   TABLE_SI0_REQ               ; fetch request
    CMP     A, [HUB_bRequest]           ; = bRequest? (match)
    JZ      SI0_REQ_FOUND               ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      SI0_REQ_NOT_FOUND           ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     SI0_LOOK                    ; look at next request for match
SI0_REQ_NOT_FOUND:
    JMP     HUB_EP0_STALL               ; invalid bRequest
SI0_REQ_FOUND:
    MOV     A, X                        ; addr*2
    ASL                                 ; *2
    JACC    JMP_TABLE_SI0_REQ           ; jump to request processor

;........................................................................
SET_DEVICE_ADDRESS:
; [1] 9.4.6 Set Address
; Sets the device address for all future device accesses
; Bumps STATE_HUB to ADDRESS if = DEFAULT and address != 0
;........................................................................

    MOV     A, [HUB_wValue_MSB]         ; ensure wValue MSB = 0
    CMP     A, 0
    JNZ     HUB_EP0_STALL               ; if not, exit
    MOV     A, [HUB_wValue_LSB]
    CMP     A, 0                        ; valid address (>0) ?
    JZ      HUB_EP0_STALL               ; if not, exit
    OR      A, ~MASK_ADDRESS            ; set Device Address Enable
    MOV     [HUB_ASSIGNED_ADDRESS], A   ; save address value

; Unique case for SETUP: MUST wait for IN to Addr 0x00 before setting address
SDA_IN_WAIT:                            ; wait for IN and ACK before setting Address
    unlock  REG_EP_B0_MODE, NAK_OUT_STATUS_IN
SDA_IN_LOOP:
    read    REG_EP_B0_MODE              ; read the mode register
    AND     A,(MASK_TOKEN + MASK_ACK)
    JZ      SDA_IN_LOOP
    CMP     A, (IN_TOKEN + MASK_ACK)     ; token = IN?
    JNZ     TASK_HUB_EP0_END            ; if not, get out

    MOV     A, [HUB_ASSIGNED_ADDRESS]   ; get address
    write_A REG_B_ADDRESS               ; update USB device B address
    MOV     A, HUB_ADDRESS              ; bump hub state to ADDRESS
    MOV     [STATE_HUB], A

    unlock  REG_EP_B0_MODE, STALL
    JMP     TASK_HUB_EP0_END            ; fall out!

;........................................................................
SET_DEVICE_CONFIGURATION:
; [1] 9.4.7 Set Configuration
; Sets the device configuration
; [wValue] Configuration Value (from host)
;
; Set the configuration of the device to either unconfigured (0) or
; configured (1) based on wValue in the SETUP packet.  According to
; the USB spec (page 178), a Set Configuration also clears the endpoint
; stall condition and re-initializes endpoints using data 0/1 toggle to
; Data0.
;........................................................................

	
	MOV 	A, [HUB_wValue_MSB] 
	MOV	[HUB_CONFIG_MSB], A 
	MOV 	A, [HUB_wValue_LSB] 
	MOV	[HUB_CONFIG_LSB], A 
; CONFIG values of 0 and 1 are acceptable. Other values are not acceptable

	CMP	A, UNCONFIGURED		; Check if config value is "0"
	JZ	CONFIG_CONTINUE
	CMP	A, CONFIGURED		; Check if config value is "1"
	JNZ	HUB_NOT_CONFIGURED_STATE ; otherwise, drop out

CONFIG_CONTINUE:
	read	REG_EP_B0_COUNT		; clear the DATA0/1 flag, based on sepc.
	AND	A,~MASK_DATATOGGLE	; clear the bit
	MOV	[TEMP], A
	unlock	REG_EP_B0_COUNT, [TEMP]	; save the data toggle bit
	MOV	A, 0			; Next, clear the stall condition
	MOV	[FLAG_STALL], A		; clear the FLAG_STALL
 	MOV 	A, [HUB_wValue_LSB] 	; check the CONFIG value, if set HUB_STATE
	MOV	[HUB_CONFIG_LSB], A 	; if set HUB_STATE and enable hub_ep1_int
	CMP	A, CONFIGURED		; Check if config value is "1"
	JNZ	HUB_NOT_CONFIGURED_STATE
    MOV     A, HUB_CONFIGURED           ; bump hub state to configured
    MOV     [STATE_HUB] , A             ; !!! enumeration complete !!!
    unlock_EP  REG_EP_B1_MODE, NAK_IN   ; configured, start NAKing on EP1
    enable_hub_ep1_int                  ; enable the HUB EP1 interrupt
    write   0h, REG_PORT_FORCE_LOW1_4   ; remove SE0 forcing 
    MOV     A,MAX_EMBEDDED_PORTS
    CMP     A,5
    JC      SKIP_NFL_HIGH
    write   0h, REG_PORT_FORCE_LOW5_7
SKIP_NFL_HIGH:
    JMP     NO_DATA_CONTROL

HUB_NOT_CONFIGURED_STATE:
    unlock_EP  REG_EP_B1_MODE, DISABLED ;unconfigured, don't respond on EP1
    write   0ffh,REG_PORT_FORCE_LOW1_4
    MOV     A,MAX_EMBEDDED_PORTS
    CMP     A,5
    JC      SKIP_FL_HIGH
    write   0ffh,REG_PORT_FORCE_LOW5_7
SKIP_FL_HIGH:
    JMP     NO_DATA_CONTROL            ; fall out!

;========================================================================
PROC_ST_IN_1:
; Standard Interface specific host-to-device (IN) request
; [bRequest] CLEAR_FEATURE, SET_FEATURE, SET_INTERFACE
;========================================================================
;-- table search for HUB_bRequest
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
SI1_LOOK:
    INDEX   TABLE_SI1_REQ               ; fetch request
    CMP     A, [HUB_bRequest]           ; = bRequest? (match)
    JZ      SI1_REQ_FOUND               ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      SI1_REQ_NOT_FOUND           ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     SI1_LOOK                    ; look at next request for match
SI1_REQ_NOT_FOUND:
    JMP     HUB_EP0_STALL               ; invalid bRequest
SI1_REQ_FOUND:
    MOV     A, X                        ; addr*2
    ASL                                 ; *2
    JACC    JMP_TABLE_SI1_REQ           ; jump to request processor

;========================================================================
PROC_ST_IN_2:
; Standard Endpoint specific host-to-device (IN) request
; [bRequest] CLEAR_FEATURE, SET_FEATURE
;========================================================================
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
SI2_LOOK:
    INDEX   TABLE_SI2_REQ               ; fetch request
    CMP     A, [HUB_bRequest]           ; = bRequest? (match)
    JZ      SI2_REQ_FOUND               ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      SI2_REQ_NOT_FOUND           ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     SI2_LOOK                    ; look at next request for match
SI2_REQ_NOT_FOUND:
    JMP     HUB_EP0_STALL               ; invalid bRequest
SI2_REQ_FOUND:
    MOV     A, X                        ; addr*2
    ASL                                 ; *2
    JACC    JMP_TABLE_SI2_REQ           ; jump to request processor

;........................................................................
CLEAR_ENDPOINT_FEATURE:
; Recipient --> endpoint
; Only ENDPOINT_STALL supported
;........................................................................
    MOV     A, [HUB_wValue_MSB]
    CMP     A, 0
    JNZ     HUB_EP0_STALL
    MOV     A, [HUB_wValue_LSB]
    CMP     A, ENDPOINT_STALL
    JZ      CLEAR_ENDPOINT_STALL
    JMP     HUB_EP0_STALL

CLEAR_ENDPOINT_STALL:
    MOV     A, [HUB_wIndex_LSB]
    AND     A, MASK_ENDPOINT
    JZ	    HUB_EP0_STALL

;CLEAR_HUB_EP0_STALL:
;    JMP     CLEAR_ENDPOINT_FEATURE_END

CLEAR_HUB_EP1_STALL:
	MOV	A, 0			; 
	MOV	[FLAG_STALL], A		; clear the STALL flag
    unlock_EP  REG_EP_B1_MODE, NAK_IN   ; NAK IN's
    unlock  REG_EP_B1_COUNT, 80h        ; preset data1 bit for data0 on next data

CLEAR_ENDPOINT_FEATURE_END:
    JMP     NO_DATA_CONTROL            ; fall out!

;........................................................................
SET_ENDPOINT_FEATURE:
; Recipient --> endpoint
; Only ENDPOINT_STALL supported
;........................................................................
    MOV     A, [HUB_wValue_MSB]
    CMP     A, 0
    JNZ     HUB_EP0_STALL		; EP0 is stalled
    MOV     A, [HUB_wValue_LSB]
    CMP     A, ENDPOINT_STALL
    JNZ      HUB_EP0_STALL		

;........................................................................
SET_HUB_EP1_STALL:
;........................................................................

	MOV	A, 1			; 
	MOV	[FLAG_STALL], A		; set the STALL flag
	unlock_EP  REG_EP_B1_MODE, STALL
    JMP     NO_DATA_CONTROL            ; fall out!

;========================================================================
PROC_ST_OUT_0:
; Standard Device specific device-to-host (OUT) request
; [bRequest] GET_CONFIGURATION, GET_DESCRIPTOR, GET_STATUS
;========================================================================
;-- table search for HUB_bRequest
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
SO0_LOOK:
    INDEX   TABLE_SO0_REQ               ; fetch request
    CMP     A, [HUB_bRequest]           ; = bRequest? (match)
    JZ      SO0_REQ_FOUND               ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      SO0_REQ_NOT_FOUND           ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     SO0_LOOK                    ; look at next request for match
SO0_REQ_NOT_FOUND:
    JMP     HUB_EP0_STALL               ; invalid bRequest
SO0_REQ_FOUND:
    MOV     A, X                        ; addr*2
    ASL                                 ; *2
    JACC    JMP_TABLE_SO0_REQ           ; jump to request processor

;........................................................................
GET_DEVICE_CONFIGURATION:
;........................................................................
	MOV	A, [HUB_CONFIG_LSB]
	MOV	[FIFO_EP_B0],A
    MOV A,1
    MOV [HUB_BYTE_COUNT],A
    JMP HUB_ACK_IN_WITH_COUNT

;........................................................................
GET_DESCRIPTOR_REQUEST:                 ; Device, Configuration, String, Interface or Endpoint?
;........................................................................
    MOV     A, [HUB_wValue_MSB]         ; MSB is type, LSB is index
    CMP     A, DT_DEVICE                ; Device descriptor type?
    JZ      GET_DEVICE_DESCRIPTOR
    CMP     A, DT_CONFIGURATION         ; Configuration descriptor type?
    JZ      GET_CONFIG_DESCRIPTOR
    CMP     A,DT_STRING
    JZ      GET_STRING_DESCRIPTOR
    JMP     HUB_EP0_STALL               ; invalid wValue        ;*? verify!
;************************
GET_STRING_DESCRIPTOR:
    MOV     A, [HUB_wValue_LSB]         ; MSB is type, LSB is index
    CMP     A,STRING_LANGUAGE
    jnz     GSD_MAN
    MOV     A, (HUB_LANGUAGE_STRING_END - HUB_LANGUAGE_STRING); get num bytes to send
    MOV     [HUB_DATA_COUNT], A         ; save the count
    MOV     A, (HUB_LANGUAGE_STRING - TABLE_DESCRIPTOR) ; get start index
    JMP     SEND_DESCRIPTOR             ; send the device descriptor
GSD_MAN:
    CMP     A,STRING_MANUFACTURER
    jnz     GSD_PRODUCT
    MOV     A, (HUB_MANUFACTURER_STRING_END - HUB_MANUFACTURER_STRING); get num bytes to send
    MOV     [HUB_DATA_COUNT], A         ; save the count
    MOV     A, (HUB_MANUFACTURER_STRING - TABLE_DESCRIPTOR) ; get start index
    JMP     SEND_DESCRIPTOR             ; send the device descriptor
GSD_PRODUCT:
    CMP     A,STRING_PRODUCT
    JNZ     HUB_EP0_STALL
    MOV     A, (HUB_PRODUCT_STRING_END - HUB_PRODUCT_STRING); get num bytes to send
    MOV     [HUB_DATA_COUNT], A         ; save the count
    MOV     A, (HUB_PRODUCT_STRING - TABLE_DESCRIPTOR) ; get start index
    JMP     SEND_DESCRIPTOR             ; send the device descriptor

;************************
GET_DEVICE_DESCRIPTOR:

    MOV     A, 0                        ; index the device descriptor length
	INDEX   HUB_DEVICE_DESCRIPTOR
    MOV     [HUB_DATA_COUNT], A         ; save the device descriptor length to DATA_COUNT
    MOV     A, (HUB_DEVICE_DESCRIPTOR - TABLE_DESCRIPTOR) ; get start index
    JMP     SEND_DESCRIPTOR             ; send the device descriptor

;************************
GET_CONFIG_DESCRIPTOR:

	MOV     A, (HUB_CONFIG_DESCRIPTOR_END - HUB_CONFIG_DESCRIPTOR); get num bytes to send
    MOV     [HUB_DATA_COUNT], A         ; save the count
	MOV     A, (HUB_CONFIG_DESCRIPTOR - TABLE_DESCRIPTOR) ; get start index

SEND_DESCRIPTOR:                        ; send the descriptors
    MOV     [HUB_DATA_START], A         ; save start index
    CALL    GET_DESCRIPTOR_LENGTH       ; correct the descriptor length
    CALL    SEND_BUFFER                 ; copy first eight bytes to FIFO
    JMP     HUB_ACK_IN_WITH_DATA

;........................................................................
GET_DEVICE_STATUS:
SIZE_DEVICE_STATUS:         EQU 2
;........................................................................
    MOV     A, 0
    MOV     X, FIFO_EP_B0
    MOV     [X+1], A                    ; set status MSB = 0
    MOV     [TEMP], A                   ; use TEMP for status mask
    MOV     A, OFFSET_CONFIG_ATTRIBUTES ; offset to config attributes
    INDEX   HUB_CONFIG_DESCRIPTOR       ; get bmAttributes
    PUSH    A                           ; save
    AND     A, MASK_SELF_POWERED        ; test self powered bit
    JZ      NOT_SELF_POWERED            ; if not set go on
    MOV     A, 1                        ; else set bit 1 of status
    OR      [TEMP], A                   ; set self powered bit
NOT_SELF_POWERED:
    POP     A                           ; restore attributes
    AND     A, MASK_REMOTE_WAKEUP       ; test remote wakeup bit
    JZ      NOT_REMOTE_WAKEUP           ; if not set go on
	MOV	A, [FLAG_REMOTE_WAKEUP]	; check the remote wakeup flag
	CMP	A,0			; 
	JZ      NOT_REMOTE_WAKEUP       ; if not set go on
	    
    MOV     A, 2                        ; else set bit 2 of status
    OR      [TEMP], A                   ; set remote wakeup bit
NOT_REMOTE_WAKEUP:
    MOV     A, [TEMP]
    MOV     [X+0], A                    ; save TEMP to status LSB

    MOV     A, SIZE_DEVICE_STATUS
    MOV     [HUB_BYTE_COUNT], A         ; set TX count
    JMP     HUB_ACK_IN_WITH_COUNT       ; fall out!

;========================================================================
PROC_ST_OUT_1:
; Standard Interface specific device-to-host (OUT) request
; [bRequest] GET_INTERFACE, GET_STATUS
;========================================================================
;-- table search for HUB_bRequest
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
SO1_LOOK:
    INDEX   TABLE_SO1_REQ               ; fetch request
    CMP     A, [HUB_bRequest]           ; = bRequest? (match)
    JZ      SO1_REQ_FOUND               ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      SO1_REQ_NOT_FOUND           ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     SO1_LOOK                    ; look at next request for match
SO1_REQ_NOT_FOUND:
    JMP     HUB_EP0_STALL               ; invalid bRequest
SO1_REQ_FOUND:
    MOV     A, X                        ; addr*2
    ASL                                 ; *2
    JACC    JMP_TABLE_SO1_REQ           ; jump to request processor

;========================================================================
PROC_ST_OUT_2:
; Standard Endpoint specific device-to-host (OUT) request
; [bRequest] GET_STATUS, SYNC_FRAME
;========================================================================
    MOV     A, 0                        ; clear index
    MOV     X, A                        ; use X as the index
SO2_LOOK:
    INDEX   TABLE_SO2_REQ               ; fetch request
    CMP     A, [HUB_bRequest]           ; = bRequest? (match)
    JZ      SO2_REQ_FOUND               ; yes
    CMP     A, INVALID_REQUEST          ; end of table?
    JZ      SO2_REQ_NOT_FOUND           ; exit
    INC     X                           ; else bump index
    MOV     A, X                        ; A = index
    JMP     SO2_LOOK                    ; look at next request for match
SO2_REQ_NOT_FOUND:
    JMP     HUB_EP0_STALL               ; invalid bRequest
SO2_REQ_FOUND:
    MOV     A, X                        ; addr*2
    ASL                                 ; *2
    JACC    JMP_TABLE_SO2_REQ           ; jump to request processor

;........................................................................
SET_DEVICE_FEATURE:
; Sets or enables a specific feature
; [wValue] only DEVICE_REMOTE_WAKEUP is supported
;........................................................................
    MOV     A, [HUB_wValue_LSB]         ; get wValue (lsb only)
    CMP     A, DEVICE_REMOTE_WAKEUP     ; if LSB != DEVICE_REMOTE_WAKEUP, STALL
    JNZ     HUB_EP0_STALL

    MOV     A, OFFSET_CONFIG_ATTRIBUTES ; offset to config attributes
    INDEX   HUB_CONFIG_DESCRIPTOR       ; get bmAttributes
    AND     A, MASK_REMOTE_WAKEUP       ; test remote wakeup bit
    JZ      HUB_EP0_STALL               ; STALL if attribute not set
    set_flag    FLAG_REMOTE_WAKEUP      ; set the flag
    read    REG_PORT_SUSPEND
    OR      A,080h
    write_A REG_PORT_SUSPEND
    JMP     NO_DATA_CONTROL            ; fall out!

;........................................................................
CLEAR_DEVICE_FEATURE:
; [1] 9.4.1 Clear Feature
; Clears or disables a specific feature
; [wValue] only DEVICE_REMOTE_WAKEUP is supported
;........................................................................
    MOV     A, [HUB_wValue_LSB]         ; get wValue (lsb only)
    CMP     A, DEVICE_REMOTE_WAKEUP     ; if LSB != DEVICE_REMOTE_WAKEUP, STALL
    JNZ     HUB_EP0_STALL

    MOV     A, OFFSET_CONFIG_ATTRIBUTES ; offset to config attributes
    INDEX   HUB_CONFIG_DESCRIPTOR       ; get bmAttributes
    AND     A, MASK_REMOTE_WAKEUP       ; test remote wakeup bit
    JZ      HUB_EP0_STALL               ; STALL if attribute not set
    clr_flag    FLAG_REMOTE_WAKEUP      ; clear the flag
    read    REG_PORT_SUSPEND
    AND      A,07fh
    write_A REG_PORT_SUSPEND
    JMP     NO_DATA_CONTROL            ; fall out!
;........................................................................
GET_ENDPOINT_STATUS:
; Section 9.4.5 of the USB Spec, Get Status for an Endpoint
;........................................................................
SIZE_ENDPOINT_STATUS:         EQU 2 	; Host requires 2 bytes

	MOV	A, 0			; preapre to load the MS byte of status
	MOV	X, FIFO_EP_B0		; Initilize index to point to EP_B0 fifo
	MOV	[X+1], A
	MOV	A, [FLAG_STALL]
	MOV	[X+0], A
   	MOV     A, SIZE_ENDPOINT_STATUS         
  	MOV     [HUB_BYTE_COUNT], A         ; set TX count
    JMP     HUB_ACK_IN_WITH_COUNT       ; fall out!


;........................................................................
SET_INTERFACE_INTERFACE:
;........................................................................
    JMP     NO_DATA_CONTROL            ; fall out!
;........................................................................
;........................................................................
GET_INTERFACE_INTERFACE:
; not supported  ;*?
;........................................................................
;........................................................................
;........................................................................
SIZE_GET_INTERFACE:         EQU 1 	; Host requires 1 byte

	MOV	A, 0			; prepare to load alternate setting (always 0)
	MOV	X, FIFO_EP_B0		; Initilize index to point to EP_B0 fifo
	MOV	[X+0], A
   	MOV     A, SIZE_GET_INTERFACE        
  	MOV     [HUB_BYTE_COUNT], A         ; set TX count
    	JMP     HUB_ACK_IN_WITH_COUNT       ; fall out!
;........................................................................
SET_DEVICE_DESCRIPTOR:
; (optional, not supported)
;........................................................................
;........................................................................
CLEAR_INTERFACE_FEATURE:
; not supported
;........................................................................
;........................................................................
SET_INTERFACE_FEATURE:
; not supported
;........................................................................
;........................................................................
GET_INTERFACE_STATUS:
; not supported  ;*?
;........................................................................
;........................................................................
SYNCH_FRAME_ENDPOINT:
; not supported (supported only if pipe is isochronous)
;........................................................................
    JMP     HUB_EP0_STALL               ; invalid request

; NOTE: See JMP_TBL.ASM for JMP tables and Lookup tables for this module

; END HUB_EP0S.ASM -------------------------------------------------------