;------------------------------------------------------------------------
; JMP_TBL.ASM
;------------------------------------------------------------------------
; Rom JMP and Lookup tables for Hub message parsing
;------------------------------------------------------------------------
;
; Assembler note:
; Cannot allow XPAGE instructions to fall within the JMP instructions of a
; jump table. As implemented in this application the JMP will fail with the
; additional address offset due to the XPAGE instruction.

    XPAGEON
;------------------------------------------------------------------------
; ROM Jump Tables for HUB_EP0.ASM
;------------------------------------------------------------------------
JMP_TABLE_REQUEST_TYPE:                 ; Jump table for request type processors
    JMP PROC_ST_IN_0                    ; Standard Request Type (ST) processors
    JMP PROC_ST_IN_1                    ; (see HUB_EP0S.ASM)
    JMP PROC_ST_IN_2
    JMP PROC_ST_OUT_0
    JMP PROC_ST_OUT_1
    JMP PROC_ST_OUT_2
    JMP PROC_HT_IN_0                    ; Hub Class Specific Request Type processors
    JMP PROC_HT_IN_1                    ; Hub Request Type (HT) processors
    JMP PROC_HT_OUT_0                   ; (see HUB_EP0C.ASM)
    JMP PROC_HT_OUT_1

;------------------------------------------------------------------------
; ROM Jump Tables for HUB_EP0C.ASM
;------------------------------------------------------------------------
JMP_TABLE_CLEAR_PORT_FEATURE:
    JMP CLEAR_PORT_CONNECTION           ; port status
    JMP CLEAR_PORT_ENABLE
    JMP CLEAR_PORT_SUSPEND
    JMP CLEAR_PORT_OVER_CURRENT
    JMP CLEAR_PORT_RESET
    JMP CLEAR_PORT_POWER
    JMP CLEAR_PORT_LOW_SPEED
    JMP CLEAR_C_PORT_CONNECTION         ; port change
    JMP CLEAR_C_PORT_ENABLE
    JMP CLEAR_C_PORT_SUSPEND
    JMP CLEAR_C_PORT_OVER_CURRENT
    JMP CLEAR_C_PORT_RESET

JMP_TABLE_SET_PORT_FEATURE:
    JMP SET_PORT_CONNECTION             ; port status
    JMP SET_PORT_ENABLE
    JMP SET_PORT_SUSPEND
    JMP SET_PORT_OVER_CURRENT
    JMP SET_PORT_RESET
    JMP SET_PORT_POWER
    JMP SET_PORT_LOW_SPEED
    JMP SET_C_PORT_CONNECTION           ; port change
    JMP SET_C_PORT_ENABLE
    JMP SET_C_PORT_SUSPEND
    JMP SET_C_PORT_OVER_CURRENT
    JMP SET_C_PORT_RESET

;------------------------------------------------------------------------
; ROM Jump Tables for HUB_EP0S.ASM
;------------------------------------------------------------------------
JMP_TABLE_SI0_REQ:
    JMP CLEAR_DEVICE_FEATURE
    JMP SET_DEVICE_ADDRESS
    JMP SET_DEVICE_CONFIGURATION
    JMP SET_DEVICE_DESCRIPTOR
    JMP SET_DEVICE_FEATURE

JMP_TABLE_SI1_REQ:
    JMP CLEAR_INTERFACE_FEATURE
    JMP SET_INTERFACE_FEATURE
    JMP SET_INTERFACE_INTERFACE

JMP_TABLE_SI2_REQ:
    JMP CLEAR_ENDPOINT_FEATURE
    JMP SET_ENDPOINT_FEATURE

JMP_TABLE_SO0_REQ:
    JMP GET_DEVICE_CONFIGURATION
    JMP GET_DESCRIPTOR_REQUEST
    JMP GET_DEVICE_STATUS

JMP_TABLE_SO1_REQ:
    JMP GET_INTERFACE_INTERFACE
    JMP GET_INTERFACE_STATUS

JMP_TABLE_SO2_REQ:
    JMP GET_ENDPOINT_STATUS
    JMP SYNCH_FRAME_ENDPOINT


    XPAGEOFF
;------------------------------------------------------------------------
; ROM Lookup Tables (Type) for HUB_EP0.ASM
;------------------------------------------------------------------------
TABLE_REQUEST_TYPE:                     ; [bmRequestType] table
    DB ST_IN_0                          ; Standard Request Types (ST)
    DB ST_IN_1
    DB ST_IN_2
    DB ST_OUT_0
    DB ST_OUT_1
    DB ST_OUT_2
    DB HT_IN_0                          ; Hub Class Specific Request Types (HT)
    DB HT_IN_1
    DB HT_OUT_0
    DB HT_OUT_1
    DB INVALID_TYPE                     ; delimit table with an INVALID type

;------------------------------------------------------------------------
; ROM Lookup Tables (Type) for HUB_EP0C.ASM
;------------------------------------------------------------------------
TABLE_PORT_FEATURE:
    DB  PORT_CONNECTION                 ; port status
    DB  PORT_ENABLE
    DB  PORT_SUSPEND
    DB  PORT_OVER_CURRENT
    DB  PORT_RESET
    DB  PORT_POWER
    DB  PORT_LOW_SPEED
    DB  C_PORT_CONNECTION               ; port change
    DB  C_PORT_ENABLE
    DB  C_PORT_SUSPEND
    DB  C_PORT_OVER_CURRENT
    DB  C_PORT_RESET
    DB  INVALID_FEATURE

;------------------------------------------------------------------------
; ROM Lookup Tables (Type) for HUB_EP0S.ASM
;------------------------------------------------------------------------
TABLE_SI0_REQ:                          ; [bRequest] table, Standard In 0 type
    DB CLEAR_FEATURE
    DB SET_ADDRESS
    DB SET_CONFIGURATION
    DB SET_DESCRIPTOR                   ; (optional)
    DB SET_FEATURE
    DB INVALID_REQUEST                  ; delimit table with an INVALID request

TABLE_SI1_REQ:                          ; [bRequest] table, Standard IN 1 type
    DB CLEAR_FEATURE
    DB SET_FEATURE
    DB SET_INTERFACE
    DB INVALID_REQUEST                  ; delimit table with an INVALID request

TABLE_SI2_REQ:                          ; [bRequest] table, Standard IN 2 type
    DB CLEAR_FEATURE
    DB SET_FEATURE
    DB INVALID_REQUEST                  ; delimit table with an INVALID request

TABLE_SO0_REQ:                          ; [bRequest] table, Standard OUT 0 type
    DB GET_CONFIGURATION
    DB GET_DESCRIPTOR
    DB GET_STATUS
    DB INVALID_REQUEST                  ; delimit table with an INVALID request

TABLE_SO1_REQ:                          ; [bRequest] table, Standard OUT 1 type
    DB GET_INTERFACE
    DB GET_STATUS
    DB INVALID_REQUEST                  ; delimit table with an INVALID request

TABLE_SO2_REQ:                          ; [bRequest] table, Standard OUT 2 type
    DB GET_STATUS
    DB SYNCH_FRAME
    DB INVALID_REQUEST                  ; delimit table with an INVALID request

    XPAGEON