;------------------------------------------------------------------------
; TIMER.ASM
; contains 1ms ISR routine
;------------------------------------------------------------------------

;------------------------------------------------------------------------
INT_128US_TIMER:
; 128us Timer Interrupt handler
;------------------------------------------------------------------------
;    INC     [COUNT_128US]
    RETI

;------------------------------------------------------------------------
INT_1MS_TIMER:
; 1.024ms Timer interrupt handler
; This interupt handles the counter for Hub Suspend detection and downstream
; port reset and resume timing.
;
; Timer counter note: PORT_RESET and PORT_RESUME counters auto increment on each pass
; through this interrupt. Clearing and testing these counters are responsibilities
; of their respective tasks.
;
HUB_SUSPEND_DETECT:     EQU 3           ; upstream port continuously idle (>3ms)
HUB_SUSPEND_LIMIT:      EQU 5           ; >5ms
PORT_RESET_LIMIT:       EQU 11          ; downstream port reset signaling (>10ms)
PORT_RESUME_LIMIT:      EQU 20          ; selective downstream resume signaling (>20ms)
PORT_RESUME_COMPLETE:   EQU PORT_RESUME_LIMIT+3+1
                                        ; +3 by spec (3ms delay to send status change),
                                        ; +1 since bumped by TASK_PORT_RESUME
SEND_RESUME_LIMIT:      EQU 2
;------------------------------------------------------------------------
    EI                                  ;allow nesting
    PUSH    A
    read    REG_STAT_CTRL               ; get USB status
    AND     A, MASK_UPSTREAM_BUS_ACTIVITY ; bus activity detected?
    JNZ     UPSTREAM_ACTIVE             ; yes, skip suspend count
    INC     [HUB_SUSPEND_COUNT]         ; bump suspend counter
    MOV     A, [HUB_SUSPEND_COUNT]      ; get count
    CMP     A, HUB_SUSPEND_DETECT       ; = limit?
    JC      INC_RESET                   ; no, go on

; upstream port idle detect
    set_flag    FLAG_HUB_SUSPEND        ; else set the suspend flag
    JMP     INC_RESET

UPSTREAM_ACTIVE:
    MOV     A, 0                        ; clear suspend counter
    MOV     [HUB_SUSPEND_COUNT], A
    read    REG_STAT_CTRL               ; clear the bus activity bit
    AND     A, ~MASK_UPSTREAM_BUS_ACTIVITY
;(BETA_BUG)*******************
    AND     A, ~MASK_BETA_BUG
;*****************************
    write_A REG_STAT_CTRL

INC_RESET:
    INC     [HUB_PORT_RESET_COUNT]      ; bump port reset counter

INC_RESUME:
    INC     [HUB_PORT_RESUME_COUNT]     ; bump port resume counter
INC_OVERCURRENT:
    MOV     A,[HUB_OVERCURRENT_COUNT]
    CMP     A,LIMIT_OVERCURRENT
    JNC     HID_SERVICE
    INC     [HUB_OVERCURRENT_COUNT]         ; bump overcurrent counter
HID_SERVICE:
;(HID) JMP to KB service routine
    test_state  STATE_HID, PORT_ENABLED    ; Has Keyboard been enabled?
    JNZ     HID_1MS_TIMER_END           ; No, skip KB service interval
    hid_1ms_service
HID_1MS_TIMER_END:


;(DEBUG) for the time being...
    feed_dog

;(HID) revisit to eliminate the CALL

    POP     A
    RETI

; END TIMER.ASM ---------------------------------------------------------